@@ -29,14 +29,13 @@
  *
  * So uClibc's *scanf functions conform to the standard, and glibc's
  * implementation doesn't for the "100ergs" case mentioned above.
+ *
+ * Sep 6, 2002
+ * Patch from Tero_Lyytikinen <tero@paravant.fi> to fix bug in matchchar case.
  */
 
+#define _ISOC99_SOURCE			/* for LLONG_MAX primarily... */
 #define _GNU_SOURCE
-#include <features.h>
-#if defined(__UCLIBC__) && !defined(__USE_ISOC99)
-#define __USE_ISOC99
-#endif
-
 #define _STDIO_UTILITY
 #include <stdio.h>
 #include <stdlib.h>
@@ -46,6 +45,7 @@
 #include <stdarg.h>
 
 #ifdef __STDIO_THREADSAFE
+#include <stdio_ext.h>
 #include <pthread.h>
 #endif /* __STDIO_THREADSAFE */
 
@@ -124,9 +124,18 @@ int vsscanf(__const char *sp, __const char *fmt, va_list ap)
 
 	string->filedes = -2;		/* for debugging */
 	string->modeflags = (__FLAG_NARROW|__FLAG_READONLY);
-	string->bufstart = string->bufrpos = (unsigned char *) ((void *) sp);
+	string->bufstart = string->bufpos = (unsigned char *) ((void *) sp);
 	string->bufgetc = (char *) ((unsigned) -1);
 
+#ifdef __STDIO_MBSTATE
+	__INIT_MBSTATE(&(string->state));
+#endif /* __STDIO_MBSTATE */
+
+#ifdef __STDIO_THREADSAFE
+	string->user_locking = 0;
+	__stdio_init_mutex(&string->lock);
+#endif
+
 	return vfscanf(string, fmt, ap);
 }
 #else  /* __STDIO_BUFFERS */
@@ -161,16 +170,18 @@ static int valid_digit(char c, char base)
 	if (base == 16) {
 		return isxdigit(c);
 	} else {
-		return (isdigit(c) && (c < '0' + base));
+		return (__isdigit(c) && (c < '0' + base));
 	}
 }
 
 extern unsigned long
 _stdlib_strto_l(register const char * __restrict str,
 				char ** __restrict endptr, int base, int sflag);
+#ifdef LLONG_MAX
 extern unsigned long long
 _stdlib_strto_ll(register const char * __restrict str,
 				 char ** __restrict endptr, int base, int sflag);
+#endif
 
 struct scan_cookie {
 	FILE *fp;
@@ -182,14 +193,9 @@ struct scan_cookie {
 	int app_ungot;
 };
 
-#ifdef __UCLIBC_HAS_LONG_LONG__
 static const char qual[] = "hl" /* "jtz" */ "Lq";
 /* char = -2, short = -1, int = 0, long = 1, long long = 2 */
 static const char qsz[] = { -1, 1,           2, 2 };
-#else
-static const char qual[] = "hl" /* "jtz" */;
-static const char qsz[] = { -1, 1,         };
-#endif
 
 #ifdef __UCLIBC_HAS_FLOATS__
 static int __strtold(long double *ld, struct scan_cookie *sc);
@@ -201,7 +207,8 @@ static const char spec[]  = "%n[csoupxXid";
 /* radix[i] <-> spec[i+5]     o   u   p   x   X  i   d */
 static const char radix[] = { 8, 10, 16, 16, 16, 0, 10 };
 
-static void init_scan_cookie(struct scan_cookie *sc, FILE *fp)
+static void init_scan_cookie(register struct scan_cookie *sc,
+							 register FILE *fp)
 {
 	sc->fp = fp;
 	sc->nread = 0;
@@ -212,7 +219,7 @@ static void init_scan_cookie(struct scan_cookie *sc, FILE *fp)
 
 /* TODO -- what about literal '\0' chars in a file??? */
 
-static int scan_getc_nw(struct scan_cookie *sc)
+static int scan_getc_nw(register struct scan_cookie *sc)
 {
 	if (sc->ungot_flag == 0) {
 		sc->ungot_char = getc(sc->fp);
@@ -226,7 +233,7 @@ static int scan_getc_nw(struct scan_cookie *sc)
 	return sc->ungot_char;
 }
 
-static int scan_getc(struct scan_cookie *sc)
+static int scan_getc(register struct scan_cookie *sc)
 {
 	if (sc->ungot_flag == 0) {
 		sc->ungot_char = getc(sc->fp);
@@ -243,7 +250,7 @@ static int scan_getc(struct scan_cookie *sc)
 	return sc->ungot_char;
 }
 
-static void scan_ungetc(struct scan_cookie *sc)
+static void scan_ungetc(register struct scan_cookie *sc)
 {
 	if (sc->ungot_flag != 0) {
 		assert(sc->width < 0);
@@ -258,7 +265,7 @@ static void scan_ungetc(struct scan_cookie *sc)
 	}
 }
 
-static void kill_scan_cookie(struct scan_cookie *sc)
+static void kill_scan_cookie(register struct scan_cookie *sc)
 {
 	if (sc->ungot_flag) {
 		ungetc(sc->ungot_char,sc->fp);
@@ -269,30 +276,20 @@ static void kill_scan_cookie(struct scan_cookie *sc)
 	}
 }
 
-int vfscanf(fp, format, ap)
-FILE *fp;
-const char *format;
-va_list ap;
+int vfscanf(FILE *fp, const char *format, va_list ap)
 {
-#ifdef __UCLIBC_HAS_LONG_LONG__
 #define STRTO_L_(s,e,b,sf) _stdlib_strto_ll(s,e,b,sf)
 #define MAX_DIGITS 64
 #define UV_TYPE unsigned long long
 #define V_TYPE long long
-#else
-#define STRTO_L_(s,e,b,sf) _stdlib_strto_l(s,e,b,sf)
-#define MAX_DIGITS 32
-#define UV_TYPE unsigned long
-#define V_TYPE long
-#endif
 #ifdef __UCLIBC_HAS_FLOATS__
 	long double ld;
 #endif
 	UV_TYPE uv;
 	struct scan_cookie sc;
-	unsigned const char *fmt;
+	register unsigned const char *fmt;
 	const char *p;
-	unsigned char *b;
+	register unsigned char *b;
 	void *vp;
 	int cc, i, cnt;
 	signed char lval;
@@ -317,7 +314,7 @@ va_list ap;
 				store = 0;
 				++fmt;
 			}
-			for (i = 0 ; isdigit(*fmt) ; sc.width = i) {
+			for (i = 0 ; __isdigit(*fmt) ; sc.width = i) {
 				i = (i * 10) + (*fmt++ - '0'); /* Get specified width. */
 			}
 			for (i = 0 ; i < sizeof(qual) ; i++) { /* Optional qualifier. */
@@ -342,8 +339,9 @@ va_list ap;
 					goto nextfmt;
 				}
 				if (p-spec > 3) { /* skip white space if not c or [ */
-					while (isspace(scan_getc_nw(&sc)))
-						{}
+					do {
+						i = scan_getc_nw(&sc);
+					} while (__isspace(i));
 					scan_ungetc(&sc);
 				}
 				if (p-spec < 5) { /* [,c,s - string conversions */
@@ -355,7 +353,7 @@ va_list ap;
 						}
 					}
 					for (i=0 ; i<= UCHAR_MAX ; i++) {
-						scanset[i] = ((*p == 's') ? (isspace(i) == 0) : 0);
+						scanset[i] = ((*p == 's') ? (__isspace(i) == 0) : 0);
 					}
 					if (*p == '[') { /* need to build a scanset */
 						if (*++fmt == '^') {
@@ -483,15 +481,12 @@ va_list ap;
 						vp = va_arg(ap, void *);
 						switch (lval) {
 							case 2:	/* If no long long, treat as long . */
-#ifdef __UCLIBC_HAS_LONG_LONG__
 								*((unsigned long long *)vp) = uv;
 								break;
-#endif
 							case 1:
 #if ULONG_MAX == UINT_MAX
 							case 0:	/* int and long int are the same */
 #endif
-#ifdef __UCLIBC_HAS_LONG_LONG__
 								if (usflag) {
 									if (uv > ULONG_MAX) {
 										uv = ULONG_MAX;
@@ -501,7 +496,6 @@ va_list ap;
 								} else if (((V_TYPE)uv) < LONG_MIN) {
 									uv = (UV_TYPE) LONG_MIN;
 								}
-#endif
 								*((unsigned long *)vp) = (unsigned long)uv;
 								break;
 #if ULONG_MAX != UINT_MAX
@@ -518,7 +512,7 @@ va_list ap;
 								*((unsigned int *)vp) = (unsigned int)uv;
 								break;
 #endif
-							case -1:
+							case (signed char)(-1):
 								if (usflag) {
 									if (uv > USHRT_MAX) {
 										uv = USHRT_MAX;
@@ -530,7 +524,7 @@ va_list ap;
 								}
 								*((unsigned short *)vp) = (unsigned short)uv;
 								break;
-							case -2:
+							case (signed char)(-2):
 								if (usflag) {
 									if (uv > UCHAR_MAX) {
 										uv = UCHAR_MAX;
@@ -583,12 +577,14 @@ va_list ap;
 			}
 			/* Unrecognized specifier! */
 			goto RETURN_cnt;
-		} if (isspace(*fmt)) {	/* Consume all whitespace. */
-			while (isspace(scan_getc_nw(&sc)))
-				{}
+		} if (__isspace(*fmt)) {	/* Consume all whitespace. */
+			do {
+				i = scan_getc_nw(&sc);
+			} while (__isspace(i));
 		} else {				/* Match the current fmt char. */
 		matchchar:
 			if (scan_getc_nw(&sc) != *fmt) {
+				scan_ungetc(&sc);
 				goto done;
 			}
 			scan_getc_nw(&sc);
@@ -654,7 +650,7 @@ int __strtold(long double *ld, struct scan_cookie *sc)
     since_decimal = INT_MIN;
 
  LOOP:
-    while (isdigit(c)) {		/* Process string of digits. */
+    while (__isdigit(c)) {		/* Process string of digits. */
 		++since_decimal;
 		if (num_digits < 0) {	/* First time through? */
 			++num_digits;		/* We've now seen a digit. */
@@ -701,7 +697,7 @@ int __strtold(long double *ld, struct scan_cookie *sc)
 
 		num_digits = 0;
 		exponent_temp = 0;
-		while (isdigit(c)) {	/* Process string of digits. */
+		while (__isdigit(c)) {	/* Process string of digits. */
 			if (exponent_temp < MAX_ALLOWED_EXP) { /* overflow check */
 				exponent_temp = exponent_temp * 10 + (c - '0');
 			}
